import React, { useState, useLayoutEffect, useRef, useEffect, useMemo, useCallback } from 'react';
import { parseMetadata, noop } from '@highlight-ui/utils-commons';
import { DragIcon, DragAndDropList } from '@highlight-ui/drag-and-drop';
import classnames from 'classnames';
import debounce from 'lodash.debounce';
import { Typography } from '@highlight-ui/typography';
import { Icon } from '@highlight-ui/icon';
import { Checkbox } from '@highlight-ui/checkbox';
/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function () {
  __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

function __rest(s, e) {
  var t = {};

  for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) t[p] = s[p];

  if (s != null && typeof Object.getOwnPropertySymbols === "function") for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
    if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
  }
  return t;
}

function __spreadArray(to, from, pack) {
  if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
    if (ar || !(i in from)) {
      if (!ar) ar = Array.prototype.slice.call(from, 0, i);
      ar[i] = from[i];
    }
  }
  return to.concat(ar || Array.prototype.slice.call(from));
}

var styles$4 = {
  "container": "CollapsibleRaw-module__3sgxK-ar__v3-3-4",
  "disabled": "CollapsibleRaw-module__2YcraYrJ__v3-3-4",
  "flex": "CollapsibleRaw-module__1xz1myMQ__v3-3-4",
  "togglerWrapper": "CollapsibleRaw-module__CMKsHAQR__v3-3-4",
  "togglerWrapperCursor": "CollapsibleRaw-module__xkufQmIR__v3-3-4",
  "contentWrapper": "CollapsibleRaw-module__1PVdOLPw__v3-3-4"
};

function useDynamicHeight(observeElement) {
  var _a = useState(0),
      height = _a[0],
      setHeight = _a[1];

  useLayoutEffect(function () {
    setHeight(observeElement ? observeElement.scrollHeight : 0);
    var debouncedResizeHandler = debounce(function () {
      setHeight(observeElement ? observeElement.scrollHeight : 0);
    }, 100);
    var mutationObserver = new MutationObserver(function () {
      setHeight(observeElement ? observeElement.scrollHeight : 0);
    });
    var resizeObserver = new ResizeObserver(function () {
      debouncedResizeHandler();
    });

    if (observeElement) {
      mutationObserver.observe(observeElement, {
        childList: true,
        subtree: true
      });
      resizeObserver.observe(observeElement);
    }

    return function () {
      mutationObserver.disconnect();
      resizeObserver.disconnect();
    };
  }, [observeElement]);
  return height;
} // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types


var Collapsible = function (_a) {
  var _b, _c;

  var action = _a.action,
      children = _a.children,
      className = _a.className,
      _d = _a.classNames,
      classNames = _d === void 0 ? {
    toggler: '',
    togglerComponent: '',
    contentWrapper: ''
  } : _d,
      tabIndex = _a.tabIndex,
      _e = _a.disabled,
      disabled = _e === void 0 ? false : _e,
      metadata = _a.metadata,
      togglerComponent = _a.togglerComponent,
      _f = _a.initiallyExpanded,
      initiallyExpanded = _f === void 0 ? false : _f,
      onExpansionChange = _a.onExpansionChange;

  var _g = useState(initiallyExpanded),
      isExpanded = _g[0],
      setIsExpanded = _g[1];

  var _h = useState(),
      contentElement = _h[0],
      setContentElement = _h[1];

  var containerRef = useRef(); // this hook must be here to support dynamic response to content DOM manipulations

  var contentHeight = useDynamicHeight(contentElement);
  useEffect(function () {
    setIsExpanded(initiallyExpanded);
  }, [initiallyExpanded]);

  var toggleCollapsible = function () {
    if (disabled) return;
    setIsExpanded(function (prevIsExpanded) {
      return !prevIsExpanded;
    });

    if (onExpansionChange) {
      onExpansionChange(!isExpanded);
    }
  };

  var isSpaceOrEnter = function (e) {
    return [' ', 'Enter'].includes(e.key);
  };

  var isContainerTargeted = function (e) {
    return e.target === containerRef.current;
  };

  return /*#__PURE__*/React.createElement("div", __assign({
    className: classnames(styles$4.container, (_b = {}, _b[styles$4.disabled] = disabled, _b), className)
  }, parseMetadata(metadata), {
    "aria-expanded": String(isExpanded)
  }, typeof tabIndex !== 'undefined' ? {
    tabIndex: tabIndex
  } : {}, {
    role: "button",
    onKeyDown: function (e) {
      return isSpaceOrEnter(e) && isContainerTargeted(e) && e.preventDefault();
    },
    onKeyUp: function (e) {
      return isSpaceOrEnter(e) && toggleCollapsible();
    },
    ref: containerRef
  }), /*#__PURE__*/React.createElement("div", {
    className: classnames(styles$4.flex, classNames.toggler)
  }, /*#__PURE__*/React.createElement("div", {
    className: classnames(styles$4.togglerWrapper, classNames.togglerComponent, (_c = {}, _c[styles$4.togglerWrapperCursor] = !disabled, _c)),
    onClick: toggleCollapsible,
    role: "button"
  }, togglerComponent), action), /*#__PURE__*/React.createElement("fieldset", {
    className: classnames(styles$4.contentWrapper, classNames.contentWrapper),
    style: {
      maxHeight: isExpanded ? contentHeight : 0
    },
    ref: setContentElement,
    role: "presentation",
    onKeyUp: function (e) {
      return e.stopPropagation();
    },
    disabled: !isExpanded
  }, children));
};

var styles$3 = {
  "title": "AccordionSectionTitle-module__1eKyBb_Y__v3-3-4",
  "titleArrow": "AccordionSectionTitle-module__1f8_Zrsl__v3-3-4",
  "checkBoxWrapper": "AccordionSectionTitle-module__3KgaLM3E__v3-3-4",
  "checkBox": "AccordionSectionTitle-module__11jYUdCa__v3-3-4"
};

var AccordionSectionTitle = function AccordionSectionTitle(_a) {
  var _b = _a.disabled,
      disabled = _b === void 0 ? false : _b,
      section = _a.section,
      selectable = _a.selectable,
      _c = _a.selectedSections,
      selectedSections = _c === void 0 ? [] : _c,
      _d = _a.onSelectedSectionsChange,
      onSelectedSectionsChange = _d === void 0 ? noop : _d,
      dragRef = _a.dragRef;

  var handleOnCheck = function (e) {
    var isChecked = e.target.checked;
    var newList = isChecked ? __spreadArray(__spreadArray([], selectedSections, true), [section], false) : selectedSections.filter(function (sec) {
      return sec.key !== section.key;
    });
    onSelectedSectionsChange(newList);
  };

  var accordionSectionLabel = typeof section.title === 'string' ? /*#__PURE__*/React.createElement(Typography, {
    component: "span",
    token: "heading-small"
  }, section.title) : /*#__PURE__*/React.createElement(React.Fragment, null, section.title);
  return /*#__PURE__*/React.createElement("div", {
    className: styles$3.title
  }, dragRef && /*#__PURE__*/React.createElement(DragIcon, {
    dragRef: dragRef,
    disabled: disabled
  }), /*#__PURE__*/React.createElement(Icon, {
    name: "angle-down",
    size: 16,
    type: "solid",
    color: disabled ? 'icon-disabled' : 'icon-subdued',
    className: styles$3.titleArrow
  }), selectable &&
  /*#__PURE__*/
  // eslint-disable-next-line jsx-a11y/no-static-element-interactions
  React.createElement("div", {
    className: styles$3.checkBoxWrapper,
    onClick: function (e) {
      return e.stopPropagation();
    },
    onKeyUp: function (e) {
      return e.stopPropagation();
    }
  }, /*#__PURE__*/React.createElement(Checkbox, {
    checked: selectedSections.some(function (sec) {
      return sec.key === section.key;
    }),
    readOnly: true,
    onChange: handleOnCheck,
    className: styles$3.checkBox,
    disabled: disabled
  })), accordionSectionLabel);
};

var styles$2 = {
  "sectionActions": "AccordionSectionItem-module__1fH1OGV7__v3-3-4",
  "section": "AccordionSectionItem-module__2GD6txPY__v3-3-4",
  "sectionToggler": "AccordionSectionItem-module__pV1ny0Du__v3-3-4",
  "sectionTogglerComponent": "AccordionSectionItem-module__1at7oYIp__v3-3-4",
  "sectionContent": "AccordionSectionItem-module__lg2-A6-f__v3-3-4"
};

var AccordionSectionItem = function (_a) {
  var _b, _c;

  var dragRef = _a.dragRef,
      section = _a.section,
      selectable = _a.selectable,
      onSelectedSectionsChange = _a.onSelectedSectionsChange,
      selectedSections = _a.selectedSections,
      initiallyExpanded = _a.initiallyExpanded,
      onExpansionChange = _a.onExpansionChange;
  return /*#__PURE__*/React.createElement(Collapsible, __assign({}, section.actions ? {
    action: /*#__PURE__*/React.createElement("fieldset", {
      disabled: section.disabled,
      className: classnames((_b = {}, _b[styles$2.sectionActions] = section.actions, _b)),
      role: "presentation",
      onKeyUp: function (e) {
        return e.stopPropagation();
      }
    }, section.actions)
  } : {}, {
    disabled: section.disabled,
    className: styles$2.section,
    classNames: {
      toggler: styles$2.sectionToggler,
      togglerComponent: styles$2.sectionTogglerComponent
    }
  }, section.disabled ? {} : {
    tabIndex: 0
  }, {
    togglerComponent: /*#__PURE__*/React.createElement(AccordionSectionTitle, {
      dragRef: dragRef,
      disabled: section.disabled,
      section: section,
      selectable: selectable,
      onSelectedSectionsChange: onSelectedSectionsChange,
      selectedSections: selectedSections
    }),
    initiallyExpanded: initiallyExpanded,
    onExpansionChange: onExpansionChange,
    metadata: section.metadata
  }), /*#__PURE__*/React.createElement("div", {
    className: classnames(styles$2.sectionContent, (_c = {}, _c[styles$2.selectable] = selectable, _c))
  }, section.content));
};

var defaultInitiallyOpen$1 = [];
var defaultSelectedSections$1 = [];

var DraggableAccordionWithRef = function AccordionMultiple(_a, ref) {
  var className = _a.className,
      _b = _a.initiallyOpen,
      initiallyOpen = _b === void 0 ? defaultInitiallyOpen$1 : _b,
      _c = _a.multiple,
      multiple = _c === void 0 ? false : _c,
      metadata = _a.metadata,
      onSelectedSectionsChange = _a.onSelectedSectionsChange,
      sections = _a.sections,
      _d = _a.selectable,
      selectable = _d === void 0 ? false : _d,
      _e = _a.selectedSections,
      selectedSections = _e === void 0 ? defaultSelectedSections$1 : _e,
      dragProps = _a.dragProps,
      onExpansionChange = _a.onExpansionChange;
  var openKeys = useMemo(function () {
    var firstKey = initiallyOpen.length ? [initiallyOpen[0].key] : [];
    return multiple ? initiallyOpen.map(function (_a) {
      var key = _a.key;
      return key;
    }) : firstKey;
  }, [initiallyOpen, multiple]);

  var _f = useState(openKeys),
      openSections = _f[0],
      setOpenSections = _f[1];

  var expansionChangeHandler = useCallback(function (item, index) {
    return function (isExpanded) {
      if (onExpansionChange) {
        onExpansionChange(item, isExpanded, index);
      }

      if (multiple === false) {
        setOpenSections(openSections.includes(item.key) ? [] : [item.key]);
      }
    };
  }, [multiple, onExpansionChange, openSections]);
  useEffect(function () {
    setOpenSections(openKeys);
  }, [openKeys]);
  return /*#__PURE__*/React.createElement("div", __assign({
    ref: ref,
    className: className
  }, parseMetadata(metadata)), /*#__PURE__*/React.createElement(DragAndDropList, {
    list: sections,
    dragProps: dragProps,
    renderItem: function (_a) {
      var section = _a.item,
          index = _a.index,
          dragRef = _a.dragRef;
      return /*#__PURE__*/React.createElement(AccordionSectionItem, {
        dragRef: dragRef,
        section: section,
        selectable: selectable,
        initiallyExpanded: openSections.includes(section.key),
        onExpansionChange: expansionChangeHandler(section, index),
        onSelectedSectionsChange: onSelectedSectionsChange,
        selectedSections: selectedSections
      });
    }
  }));
};

DraggableAccordionWithRef.displayName = 'DraggableAccordion';
var DraggableAccordion = /*#__PURE__*/React.forwardRef(DraggableAccordionWithRef);
var styles$1 = {
  "list": "StaticAccordion-module__1_gbRDoo__v3-3-4"
};
var defaultInitiallyOpen = [];
var defaultSelectedSections = [];

var StaticAccordionWithRef = function AccordionMultiple(_a, ref) {
  var className = _a.className,
      _b = _a.initiallyOpen,
      initiallyOpen = _b === void 0 ? defaultInitiallyOpen : _b,
      _c = _a.multiple,
      multiple = _c === void 0 ? false : _c,
      metadata = _a.metadata,
      onSelectedSectionsChange = _a.onSelectedSectionsChange,
      sections = _a.sections,
      _d = _a.selectable,
      selectable = _d === void 0 ? false : _d,
      _e = _a.selectedSections,
      selectedSections = _e === void 0 ? defaultSelectedSections : _e,
      onExpansionChange = _a.onExpansionChange;
  var openKeys = useMemo(function () {
    var firstKey = initiallyOpen.length ? [initiallyOpen[0].key] : [];
    return multiple ? initiallyOpen.map(function (_a) {
      var key = _a.key;
      return key;
    }) : firstKey;
  }, [initiallyOpen, multiple]);

  var _f = useState(openKeys),
      openSections = _f[0],
      setOpenSections = _f[1];

  var expansionChangeHandler = useCallback(function (item, index) {
    return function (isExpanded) {
      if (onExpansionChange) {
        onExpansionChange(item, isExpanded, index);
      }

      if (multiple === false) {
        setOpenSections(openSections.includes(item.key) ? [] : [item.key]);
      }
    };
  }, [multiple, onExpansionChange, openSections]);
  useEffect(function () {
    setOpenSections(openKeys);
  }, [openKeys]);
  return /*#__PURE__*/React.createElement("div", __assign({
    ref: ref,
    className: className
  }, parseMetadata(metadata)), /*#__PURE__*/React.createElement("ul", {
    className: styles$1.list
  }, sections.map(function (section, index) {
    return /*#__PURE__*/React.createElement("li", {
      key: section.key
    }, /*#__PURE__*/React.createElement(AccordionSectionItem, {
      section: section,
      selectable: selectable,
      initiallyExpanded: openSections.includes(section.key),
      onExpansionChange: expansionChangeHandler(section, index),
      onSelectedSectionsChange: onSelectedSectionsChange,
      selectedSections: selectedSections
    }));
  })));
};

StaticAccordionWithRef.displayName = 'StaticAccordion';
var StaticAccordion = /*#__PURE__*/React.forwardRef(StaticAccordionWithRef);

var AccordionWithRef = function (_a, ref) {
  var dragProps = _a.dragProps,
      restProps = __rest(_a, ["dragProps"]);

  if (dragProps) {
    return /*#__PURE__*/React.createElement(DraggableAccordion, __assign({
      ref: ref,
      dragProps: dragProps
    }, restProps));
  }

  return /*#__PURE__*/React.createElement(StaticAccordion, __assign({
    ref: ref
  }, restProps));
};

var Accordion = /*#__PURE__*/React.forwardRef(AccordionWithRef);
var styles = {
  "container": "Expandable-module__NiiKVPZx__v3-3-4",
  "toggle": "Expandable-module__3wP_8MKW__v3-3-4",
  "icon": "Expandable-module__dAl8233E__v3-3-4",
  "content": "Expandable-module__2VfzsSpY__v3-3-4",
  "toggleTitle": "Expandable-module__3vadI7bS__v3-3-4",
  "disabled": "Expandable-module__1AcZ1EF2__v3-3-4"
};

var Expandable = function (_a) {
  var _b;

  var title = _a.title,
      content = _a.content,
      className = _a.className,
      _c = _a.disabled,
      disabled = _c === void 0 ? false : _c,
      _d = _a.isInitiallyExpanded,
      isInitiallyExpanded = _d === void 0 ? false : _d,
      onExpansionChange = _a.onExpansionChange,
      metadata = _a.metadata;
  return /*#__PURE__*/React.createElement(Collapsible, __assign({
    metadata: metadata,
    disabled: disabled,
    className: classnames(styles.container, className),
    classNames: {
      togglerComponent: classnames((_b = {}, _b[styles.disabled] = disabled, _b))
    }
  }, disabled ? {} : {
    tabIndex: 0
  }, {
    togglerComponent: /*#__PURE__*/React.createElement("div", {
      className: styles.toggle
    }, /*#__PURE__*/React.createElement(Icon, {
      className: styles.icon,
      type: "solid",
      name: "angle-down"
    }), /*#__PURE__*/React.createElement(Typography, {
      className: styles.toggleTitle,
      token: "heading-small",
      component: "span"
    }, title)),
    initiallyExpanded: isInitiallyExpanded,
    onExpansionChange: onExpansionChange
  }), /*#__PURE__*/React.createElement(Typography, {
    className: styles.content,
    token: "body-base",
    component: "div"
  }, content));
};

export { Accordion, DraggableAccordion, Expandable, StaticAccordion };
//# sourceMappingURL=index.js.map
